/*@@
  @file      Preloop.F
  @date      Aug 1998
  @author    Gabrielle Allen + Miguel Alcubierre
  @desc 
     Report on slicing parameters.
  @enddesc
  @version $Header: /numrelcvs/AEIThorns/BSSN_MoL/src/PreLoop.F,v 1.7 2003/05/03 12:14:35 hawke Exp $
@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"


      subroutine ADM_BSSN_evolveLapseFlag(active_slicing_handle)

c     This small routine only sets up the value of the flag "evolveLapse".
      
      USE ADM_BSSN_Scalars

      implicit none

      INTEGER active_slicing_handle
      INTEGER Einstein_GetSlicingHandle

      if ((active_slicing_handle.eq.Einstein_GetSlicingHandle("1+log")).or.
     .    (active_slicing_handle.eq.Einstein_GetSlicingHandle("harmonic")).or.
     .    (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid")).or.
     .    (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid0")).or.
     .    (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid1"))) then
         evolveLapse = .true.
      else
         evolveLapse = .false.
      endif
	
      return
      end


c     ==================================================================


      subroutine ADM_BSSN_Setup(CCTK_ARGUMENTS)

c     This routine checks if we have changed the slicing condition
c     from the previous iteration and reports the change.

      USE ADM_BSSN_Scalars

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      logical new_slice

      integer Einstein_GetSlicingHandle

      integer,save:: prev_slicing_handle = -1

      character*50 :: infoline

      CCTK_INT :: lapseindex, lapserhsindex 
      CCTK_INT :: dtlapseindex, dtlapserhsindex
      CCTK_INT :: ierr

      CCTK_INT :: MoLChangeToEvolved
      CCTK_INT :: MoLChangeToConstrained
      CCTK_INT :: MoLChangeToSaveAndRestore

c     Set slicing flag
c     ----------------

      call ADM_BSSN_evolveLapseFlag(active_slicing_handle)


c     Check if slicing has changed from previous calls
c     ------------------------------------------------

      dtlapseindex = -1

      if (prev_slicing_handle.ne.active_slicing_handle) then
         new_slice = .true.
         call CCTK_VarIndex(lapseindex,"admbase::alp")
         call CCTK_VarIndex(lapserhsindex,"adm_bssn::adm_bs_salp")
         if ( (prev_slicing_handle.eq.Einstein_GetSlicingHandle("1+log"))
     .        .or.
     .        (prev_slicing_handle.eq.Einstein_GetSlicingHandle("harmonic"))
     .        .or.
     .        (prev_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid"))
     .        .or.
     .        (prev_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid0"))
     .        .or.
     .        (prev_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid1"))
     .        .or.
     .        (active_slicing_handle.eq.Einstein_GetSlicingHandle("1+log"))
     .        .or.
     .        (active_slicing_handle.eq.Einstein_GetSlicingHandle("harmonic"))
     .        .or.
     .        (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid"))
     .        .or.
     .        (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid0"))
     .        .or.
     .        (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid1")) )
     .        then
           call CCTK_VarIndex(dtlapseindex,"adm_bssn::ADM_BS_dtalp")
           call CCTK_VarIndex(dtlapserhsindex,"adm_bssn::ADM_BS_sdtalp")
         end if
         prev_slicing_handle = active_slicing_handle
      else
         new_slice = .false.
      end if

      if (.not.new_slice) return

c     GEODESIC SLICING
c     ----------------

      if (active_slicing_handle.eq.Einstein_GetSlicingHandle("geodesic")) then

         call CCTK_INFO("Geodesic slicing (alp=1)")
         call CCTK_INFO("  lapse will not be evolved in ADM_BSSN routines")

         ierr = MoLChangeToSaveAndRestore(lapseindex)
         if (dtlapseindex > 0) then
           ierr = MoLChangeToConstrained(dtlapseindex)
         end if

c     STATIC SLICING
c     --------------

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("static")) then

         call CCTK_INFO("Static slicing (lapse fixed at initial values)")
         call CCTK_INFO("  lapse will not be evolved in ADM_BSSN routines")

         ierr = MoLChangeToConstrained(lapseindex)
         if (dtlapseindex > 0) then
           ierr = MoLChangeToConstrained(dtlapseindex)
         end if

c     BONA-MASSO TYPE HARMONIC SLICINGS
c     ---------------------------------

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("harmonic")) then

         call CCTK_INFO("Harmonic slicing")
         write(infoline,'(A11,G12.7)') '  with f = ',real(harmonic_f)
         call CCTK_INFO(infoline)

         ierr = MoLChangeToEvolved(lapseindex, lapserhsindex)
         ierr = MoLChangeToEvolved(dtlapseindex, dtlapserhsindex)

c     1+LOG SLICING
c     -------------

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("1+log")) then

	 call CCTK_INFO("1+log slicing")

         ierr = MoLChangeToEvolved(lapseindex, lapserhsindex)
         ierr = MoLChangeToEvolved(dtlapseindex, dtlapserhsindex)

c     SHOCK AVOID SLICINGS
c     --------------------

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid")) then

	 call CCTK_INFO("shockavoid slicing")

         ierr = MoLChangeToEvolved(lapseindex, lapserhsindex)
         ierr = MoLChangeToEvolved(dtlapseindex, dtlapserhsindex)

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid0")) then

	 call CCTK_INFO("shockavoid0 slicing")

         ierr = MoLChangeToEvolved(lapseindex, lapserhsindex)
         ierr = MoLChangeToEvolved(dtlapseindex, dtlapserhsindex)

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("shockavoid1")) then

	 call CCTK_INFO("shockavoid1 slicing")

         ierr = MoLChangeToEvolved(lapseindex, lapserhsindex)
         ierr = MoLChangeToEvolved(dtlapseindex, dtlapserhsindex)

c     MAXIMAL SLICING
c     ---------------

      else if (active_slicing_handle.eq.Einstein_GetSlicingHandle("maximal")) then

	 call CCTK_INFO("Maximal slicing")
         call CCTK_INFO("  lapse will not be evolved in ADM_BSSN routines")

         ierr = MoLChangeToConstrained(lapseindex)
         if (dtlapseindex > 0) then
           ierr = MoLChangeToConstrained(dtlapseindex)
         end if           

c     UNKNOWN SLICING
c     ---------------

      else

         call CCTK_INFO("Unknown slicing")
         call CCTK_INFO("  lapse will not be evolved in ADM_BSSN routines")

         ierr = MoLChangeToSaveAndRestore(lapseindex)
         if (dtlapseindex > 0) then
           ierr = MoLChangeToConstrained(dtlapseindex)
         end if           

      end if


c     END
c     ---

      return
      end
