/*@@
  @header   GAMMASOURCES_guts.h
  @date     Nov 98
  @author   Gabrielle Allen
  @desc
     Macro to calculate the combined vacuum and matter source terms
     for BS Eq. 25 for the evolution of the Gamma components.
  @enddesc
@@*/

#ifndef GAMMASOURCES_GUTS
#define GAMMASOURCES_GUTS

#include "CactusEinstein/ADMMacros/src/macro/DA_guts.h"

#include "macro/BSCHR2_guts.h"
#include "macro/UPPERA_guts.h"
#include "macro/DPHI_guts.h"
#include "macro/DTRK_guts.h"
#include "macro/BSUPPERMET_guts.h"
#include "macro/BSHYDRO_guts.h"


      GAMMASOURCES_TWOALP = 2.0D0*alp(i,j,k)


/* Do the matter source terms */
/* -------------------------- */

      GAMMASOURCES_MATTERX = - eightpi*GAMMASOURCES_TWOALP
     &   *(BSUPPERMET_UXX*BSHYDRO_SX + BSUPPERMET_UXY*BSHYDRO_SY 
     &   + BSUPPERMET_UXZ*BSHYDRO_SZ)

      GAMMASOURCES_MATTERY = - eightpi*GAMMASOURCES_TWOALP
     &   *(BSUPPERMET_UXY*BSHYDRO_SX + BSUPPERMET_UYY*BSHYDRO_SY 
     &   + BSUPPERMET_UYZ*BSHYDRO_SZ)

      GAMMASOURCES_MATTERZ = - eightpi*GAMMASOURCES_TWOALP
     &   *(BSUPPERMET_UXZ*BSHYDRO_SX + BSUPPERMET_UYZ*BSHYDRO_SY 
     &   + BSUPPERMET_UZZ*BSHYDRO_SZ)


/* Do the vacuum source terms */
/* -------------------------- */

c     GAMMA_X.

      GAMMASOURCES_TERM1  = - 2.0D0*(UPPERA_AXX*DA_DXDA
     &   + UPPERA_AXY*DA_DYDA + UPPERA_AXZ*DA_DZDA)

      GAMMASOURCES_TERM2A = BSCHR2_XXX*UPPERA_AXX + BSCHR2_XYY*UPPERA_AYY
     &   + BSCHR2_XZZ*UPPERA_AZZ + 2.0D0*(BSCHR2_XXY*UPPERA_AXY
     &   + BSCHR2_XXZ*UPPERA_AXZ + BSCHR2_XYZ*UPPERA_AYZ)

      GAMMASOURCES_TERM2B = - twothird*(BSUPPERMET_UXX*DTRK_DXDTRK
     &   + BSUPPERMET_UXY*DTRK_DYDTRK + BSUPPERMET_UXZ*DTRK_DZDTRK)

      GAMMASOURCES_TERM2C = 6.0D0*(UPPERA_AXX*DPHI_DXDPHI
     &   + UPPERA_AXY*DPHI_DYDPHI + UPPERA_AXZ*DPHI_DZDPHI)

      GAMMASOURCES_METRICX = GAMMASOURCES_TERM1 + GAMMASOURCES_TWOALP
     &   *(GAMMASOURCES_TERM2A + GAMMASOURCES_TERM2B + GAMMASOURCES_TERM2C)

c     GAMMA_Y.

      GAMMASOURCES_TERM1  = - 2.0D0*(UPPERA_AYX*DA_DXDA
     &   + UPPERA_AYY*DA_DYDA + UPPERA_AYZ*DA_DZDA)

      GAMMASOURCES_TERM2A = BSCHR2_YXX*UPPERA_AXX + BSCHR2_YYY*UPPERA_AYY
     &   + BSCHR2_YZZ*UPPERA_AZZ + 2.0D0*(BSCHR2_YXY*UPPERA_AXY
     &   + BSCHR2_YXZ*UPPERA_AXZ + BSCHR2_YYZ*UPPERA_AYZ)

      GAMMASOURCES_TERM2B = - twothird*(BSUPPERMET_UXY*DTRK_DXDTRK
     &   + BSUPPERMET_UYY*DTRK_DYDTRK + BSUPPERMET_UYZ*DTRK_DZDTRK)

      GAMMASOURCES_TERM2C = 6.0D0*(UPPERA_AXY*DPHI_DXDPHI
     &   + UPPERA_AYY*DPHI_DYDPHI + UPPERA_AYZ*DPHI_DZDPHI)

      GAMMASOURCES_METRICY = GAMMASOURCES_TERM1 + GAMMASOURCES_TWOALP
     &   *(GAMMASOURCES_TERM2A + GAMMASOURCES_TERM2B + GAMMASOURCES_TERM2C)

c     GAMMA_Z.

      GAMMASOURCES_TERM1  = - 2.0D0*(UPPERA_AXZ*DA_DXDA
     &   + UPPERA_AYZ*DA_DYDA + UPPERA_AZZ*DA_DZDA)

      GAMMASOURCES_TERM2A = BSCHR2_ZXX*UPPERA_AXX + BSCHR2_ZYY*UPPERA_AYY
     &   + BSCHR2_ZZZ*UPPERA_AZZ + 2.0D0*(BSCHR2_ZXY*UPPERA_AXY
     &   + BSCHR2_ZXZ*UPPERA_AXZ + BSCHR2_ZYZ*UPPERA_AYZ)

      GAMMASOURCES_TERM2B = - twothird*(BSUPPERMET_UXZ*DTRK_DXDTRK
     &   + BSUPPERMET_UYZ*DTRK_DYDTRK + BSUPPERMET_UZZ*DTRK_DZDTRK)

      GAMMASOURCES_TERM2C = 6.0D0*(UPPERA_AXZ*DPHI_DXDPHI
     &   + UPPERA_AYZ*DPHI_DYDPHI + UPPERA_AZZ*DPHI_DZDPHI)

      GAMMASOURCES_METRICZ = GAMMASOURCES_TERM1 + GAMMASOURCES_TWOALP
     &   *(GAMMASOURCES_TERM2A + GAMMASOURCES_TERM2B + GAMMASOURCES_TERM2C)


/* Complete source terms */
/* --------------------- */

      adm_bs_sGx(i,j,k) = GAMMASOURCES_METRICX + GAMMASOURCES_MATTERX
      adm_bs_sGy(i,j,k) = GAMMASOURCES_METRICY + GAMMASOURCES_MATTERY
      adm_bs_sGz(i,j,k) = GAMMASOURCES_METRICZ + GAMMASOURCES_MATTERZ

#endif
