 /*@@
   @file      Whisky_Minima.F90
   @date      Mon Feb 25 11:43:36 2002
   @author    
   @desc 
   Sets up the scalars used for checking the atmosphere.
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

 /*@@
   @routine    Whisky_Minima_Setup
   @date       Mon Feb 25 11:25:27 2002
   @author     Ian Hawke
   @desc 
   Set up the scalar whisky_rho_min used for checking the atmosphere.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine Whisky_Rho_Minima_Setup(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  if (rho_abs_min > 0.d0) then
    whisky_rho_min = rho_abs_min
  else
    whisky_rho_min = whisky_rho_central * rho_rel_min
  end if

end subroutine Whisky_Rho_Minima_Setup

 /*@@
   @routine    Whisky_Minima_Setup
   @date       Mon Feb 25 11:25:27 2002
   @author     Ian Hawke
   @desc 
   Set up the scalar whisky_dens_min used for checking the atmosphere.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine Whisky_Dens_Minima_Setup(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  CCTK_INT :: handle, ierr, i, j, k
  CCTK_REAL :: local_densmin, global_densmin, det

  call CCTK_ReductionHandle(handle, "minimum")

  if (handle < 0) then
    call CCTK_WARN(0,"Cannot get reduction handle for minimum operation!")
  end if

  local_densmin = 1.d10

  do k = 1+whisky_stencil, cctk_lsh(3)-whisky_stencil
    do j = 1+whisky_stencil, cctk_lsh(2)-whisky_stencil
      do i = 1+whisky_stencil, cctk_lsh(1)-whisky_stencil
        local_densmin = min(local_densmin, &
             dens(i,j,k))

      end do
    end do
  end do
  global_densmin = local_densmin 

  call CCTK_ReduceLocScalar(ierr, cctkGH, -1, handle, local_densmin, &
       global_densmin, CCTK_VARIABLE_REAL)
  
  if (ierr < 0) then
    call CCTK_WARN(0, "Reduction operation failed!")
  end if

  whisky_dens_min = global_densmin

end subroutine Whisky_Dens_Minima_Setup
